$MODXX51
;
;Pred kompilaci je nutno zadat pozadovana tel. cisla pro ulozeni do pameti
;viz. posledni radky tohoto zdrojoveho textu!!
;
;*************** SMARTPHONE KILLER x51 by NOSTALCOMP 2015 **********************
;
;Mobilni telefon z rotacaku s pomoci AT89C2051 a PGSM modulu Quectel M10
;Nutny krystal u CPU:  11.0592 MHz
;
;V teto verzi telefon pouze umi prijmout a vytocit hovor.
;Vytacet lze pouze 9-mistna vnitrostatni cisla, nebo 3-mistna tisnova cisla
;zacinajici jednickou (158,150,155,112). Rozpoznani je automaticke, dle prvni
;vytocene cislice, 1 = tisen, ostatni = 9-mistne cislo
;
;Dale je mozne volit jedno z max. 10 prednastavenych cisel (viz. konec programu)
;To se provede nejprve vytocenim 0 a nasledne pozice prednastaveneho cisla 0-9
;
;Vzdy po vytoceni posledni platne cislice blikne zelena LED a cislo je odeslano.
;
;Atmel AT89C2051 se stara o:
; seriovou komunikaci s modulem (Rx,Tx), 
; vyrobu AC napeti pro zvonek: push-pull menic s obracene zapojenym trafickem,
; cteni rotacni ciselnice
; kontrolu napeti baterie prostrednictvim vestaveneho komparatoru
; cteni spinace vidlice, cteni bitu RING, indikaci na LEDkach a dalsi
;
;Vsechny ridici signaly mezi MCU (5V) a M10 (2,8V) musi byt konvertovany!
;Zde se jedna o RX, TX a RI (RING), POWER je jiz konvertovan na PGSM modulu
;
;K napajeni je idealni Pb 6V/1,3Ah aku a dva DC-DC menice: 
;  jeden se nastavi na 4,2V pro M10 a druhy na 5V pro MCU. 
;Napeti pro trafo DC/AC menice se bere primo z akumulatoru.
;
;Po cca minute v klidu se MCU uvede do IDLE modu se snizenou spotrebou. Ale
;telefon je stale funkcni! K probuzeni MCU staci prichozi signal RING, nebo
;zvednuti sluchatka z vidlice.
;
;POZOR! U SIM karty je nutno nejprve v nejakem telofonu zakazat pozadavek na PIN
;
;Eventualni prichozi SMS je ignorovana a neni nijak signalizovana.
;
;************* porty a konfiguracni konstanty **********************************

;P1 - P1.7 volne
UBAT      EQU  P1.0        ;aktualni napeti baterie 
UREF      EQU  P1.1        ;referencni napeti z 5V
                           ;UBAT >= UREF -> P3.6 = 1 = OK bat   
TRAFO1    EQU  P1.2        ;vinuti 1 (akt. v log 0)
TRAFO2    EQU  P1.3        ;vinuti 2 (akt. v log 0)
LEDZEL    EQU  P1.4        ;zelena LED (akt. v 0)
LEDCER    EQU  P1.5        ;cervena LED (akt. v 0)
;Obe LED jsou v jednou pouzdre (dvoubarevna LED). 
;Pri soucasne aktivaci sviti oranzove!
POWER     EQU  P1.6        ;spinani napajeni GSM modulu (akt 0)

;P3
TESTMODE  EQU  P3.7        ;zap. test. rezimu (log.0) - bez prijmu z GSM modulu 
LOWBAT    EQU  P3.6        ;vystup komparatoru sledujiciho napeti baterie
                           ;bit je 1, pokud UP1.0 >= UP1.1
NATAH     EQU  P3.5        ;kontakt ciselnice NATAH spinaci
PULS      EQU  P3.4        ;kontakt ciselnice PULS rozpinaci
VIDLICE   EQU  P3.3        ;kontakt vidlice sluchatka, zvednuto=0, polozeno=1 
RING      EQU  P3.2        ;vstup RING z GSM modulu, zvoneni=0, klid=1
;P3.0 (RX) a P3.1 (TX) jsou rezervovany pro seriak!

ZSTATE    EQU  R7          ;stav zvonku 0 - 3  (vinuti1, dead, vinuti2, dead)
CIFRA     EQU  R6          ;jedna vytocena cifra z ciselnice
;R0 pouzit pro neprime adresovani pro ukladani vytaceneho tel. cisla
;R1 pouzit pro vysilani a prijem seriaku

;konstanty pro ovladani a casovani pri krystalu 11.059 MHz
PULS1     EQU  65535-250	      ;Preset citace pro puls 0.2 mS (deadtime)
PULS10    EQU  65535-9400	      ;Preset citace pro puls 10 mS (puls)

C_MASK    EQU  00110000b        ;maska kontaktu ciselnice P3.4 PULS, P3.5 NATAH
C_KLID    EQU  00100000b        ;stav kontaktu v klidu ciselnice
C_NATAH   EQU  00000000b        ;stav kontaktu pri natahu ciselnice
C_PULS    EQU  00010000b        ;stav kontaktu pri pulsu ciselnice
C_ZAK     EQU  00110000b        ;zakazany stav ciselnice, nemel by nastat, ale
                                ;je nutne ho osetrit!    

POCPRUCH  EQU  100              ;poc. pruchodu protizakmit rutinou ciselnice OK!
                                ;vyuzivano i pro osetreni kontaktu vidlice
TEL3      EQU  3                ;max. pocet vytocenych cifer na tisnove  cislo
TEL9      EQU  9                ;max. pocet vytocenych cifer na std. tel. cislo
TELBASE   EQU  010h             ;pocatecni adresa ukladani tel cisla
                                ;pozor: prepisuje registrove banky 2 a 3!!!
ZVONENICYK EQU 150              ;pocet pulperiod na interval zvoneni a pauzy 
                                ;pulperioda = cca 10 ms

; bitsegment
BSEG AT 020h
ZVON        BIT 0               ;priznak zvonku/pauzy behem zvoneni
ERRBIT      BIT 1               ;priznak chyby
ERRTIME     BIT 2               ;priznak casovani chyby
SEZNAM      BIT 3               ;priznak vyberu cisla ze seznamu
TSTMODE     BIT 4               ;priznak testovaciho modu (necte z GSM modulu)
                                ;lze vyuzit pro testovani bez pripojeneho modulu

; datasegment
DSEG AT 030h
TOF1:        DS  1              ;pom. pocitadlo pro casovani zpozdeni
TOF2:        DS  1              ;pom. pocitadlo pro casovani zpozdeni
TOF3:        DS  1              ;pom. pocitadlo pro casovani zpozdeni
COUNT:       DS  1              ;pocitadlo pruch. protizakmit smyckou ciselnice
STAVOLD:     DS  1              ;osetreny stav ciselnice - stary
STAVNEW:     DS  1              ;osetreny stav ciselnice - novy
CISOLD:      DS  1              ;neosetreny stav konkatu ciselnice - stary
CISNEW:      DS  1              ;neosetreny stav konkatu ciselnice - novy
CIFCOUNT:    DS  1              ;pocitadlo vytocenych cifer
POCCIFER:    DS  1              ;cilovy pocet cifer na vytacene cislo
ERRCODE:     DS  1              ;kod chyby (pro naslednou indikaci poctem bliku)
ERRCOUNT:    DS  1              ;citac preteceni chyboveho casovace

;*******************************************************************************
;		Zacatek programu - codesegment
;*******************************************************************************		
CSEG AT 00000h
ORG 00000h
	
start:	
      SJMP init             ;obskok preruseni   
      
ORG 00003h  
	    ;preruseni od INT 0 (RING)
      jmp inter0       
       
ORG 0000Bh  
	    ;preruseni od citace 0
      jmp citac0
      
ORG 00013h  
	    ;preruseni od INT 1 (VIDLICE)
      jmp inter1         
        
ORG 0001Bh  
 	    ;preruseni od citace 1   (nepouzito)
 	    ;jmp citac1
 	    
ORG 00020h
init: 
      mov SP,#040h          ;STACK pointer
	    mov IE,#000h		      ;zaka peruen
	    mov P1,#0FFh          ;nastav porty na H
	    mov P3,#0FFh
      clr TSTMODE           ;testovaci mod vypnut
      jb TESTMODE, init2    ;je TESTMODE na log.1?
      setb TSTMODE          ;NE:testovaci mod zapnut
init2:                      ;ANO: jedeme dal
	    clr POWER             ;zapni napajeni GSM modulu  
	    clr LEDCER            ;rozsvit cervenou LED
	    call cekej2s          ;cekej 2 sec
	    setb LEDCER           ;kratke zhasnuti cervene LED
	    call cekej250ms
	    clr LEDCER            ;a opet ji rozsvitime
      setb POWER            ;vypni napajeci impuls (mudul zustane zapnuty) 
	    call cekej2s          ;cekame na registraci do GSM site   
	    call cekej2s          ;trva to dlouho........
	    call cekej2s          ;a jinak to neslo, protoze po zapnuti dlouhou dobu
	    call cekej2s          ;modul nekomunikuje po seriaku.
	    call cekej2s          ;proste nejlepsi je pasivne pockat....  
	    call cekej2s          ;alespon si pripravime casovace:
	    
	    MOV TMOD, #00100001b    ;casovac 0 mod 1 (16b), casovac 1 mod 2 (8b)
	    MOV TCON, #00000101b    ;casovace v klidu, INT0 a INT1 na sestupnou hranu 
	    MOV PCON, #00000000b    ;SMOD = 0 (nelze adresovat bitove)
	    MOV SCON, #01000000b    ;UART 8 bit, potom povolit prijem REN=1!
	    MOV TH1, #0FDh          ;9600 Bd
	    setb TR1                ;spust casovac 1, generujici prenosovou rychlost
	    
	    call cekej2s          ;nechame casovac ustalit
	    call cekej2s          
	    setb LEDCER           ;zhasni cervenou LED
      
      mov dptr,#TXT_AT      ;vysleme obecny prikaz AT (radeji 3x)
	    call poslitext        ;aby funkce autobaud detekovala rychlost seriaku 
	    call newline
	    nop 
      nop
      mov dptr,#TXT_AT      ;vysleme obecny prikaz AT (radeji 3x)
	    call poslitext        ;aby funkce autobaud detekovala rychlost seriaku 
	    call newline
	    nop 
      nop
      mov dptr,#TXT_AT      ;vysleme obecny prikaz AT (radeji 3x)
	    call poslitext        ;aby funkce autobaud detekovala rychlost seriaku 
	    call newline
	    nop 
      nop
      mov dptr,#TXT_ATE0    ;vysleme prikaz pro vypnuti Echa z modulu
	    call poslitext        ;aby nam porad nevracel nase prikazy zpet 
	    call newline
	    nop 
      nop
	    mov dptr,#TXT_ATE0    ;posleme ho celkem 3x, 2x naslepo 
	    call poslitext        ;a potreti uz budeme cekat na odpoved OK, cimz se 
	    call newline          ;ujistime, ze modul zije 
	    nop
	    nop
	    mov ERRCODE,#1        ;pripravime kod chyby, kdyby nedosla odpoved 
	    mov dptr,#TXT_ATE0    ;potreti vysleme ATE0 
	    call poslitext        ;zde samotny text prikazu 
	    call newline          ;nasledovany ENTERem 
	    jb TSTMODE, NOTX1     ;testovaci mod?
	    setb REN              ;povolime prijem ze seriaku
	    call errstart         ;spustime casovac hlidajici max. cas na odpoved 
	    call isok             ;a cekame na OK z modulu 
	    jb ERRBIT, error      ;po prijatem O nebylo K, skoc na indikaci chyby 
	    call errstop          ;odpoved dosla ok, vypneme casovac 
	   
NOTX1:	   
	    mov ERRCODE,#2        ;pripravime kod chyby, kdyby nedosla odpoved 
	    mov dptr,#TXT_GSM     ;a zeptame se modulu, zdali je uspesne registrovan 
	    call poslitext        ;v GSM siti
	    call newline
	    jb TSTMODE, NOTX2     ;testovaci mod?
	    call errstart         ;opet nastartujeme casovac
	    call isgsm            ;cekame na odpoved. Musi obsahovat retezec 0,1 
	    jb ERRBIT, error      ;spatny retezec (asi 0,0) - nejsme v siti, chyba
	    call errstop          ;odpoved dosla ok, vypneme casovac  

NOTX2:	    
;Dalsi prikazy uz muzeme posilat naslepo
      clr REN               ;prijem uz nebudeme potrebovat, tak ho vypneme
	    mov dptr,#TXT_CHA0    ;prikaz pro prepnuti audia na poz. konektor modulu 
	    call poslitext         
	    call newline
      
      mov dptr,#TXT_ATH      ;preventivni zaveseni, kdyby se k nam nekdo cpal...
	    call poslitext
	    call newline
	    jmp trojblik           ;a jdeme cekat na dalsi pouziti naseho telefonu
	    
;******************** ERROR ****************************************************
error:                       ;rutina pro ohlaseni chyby kratkym blikanim
                             ;cervene LED. Pocet kratkych bliku odpovida ERRCODE
                             ;nasleduje delsi pauza a pak zase kratke bliky
                             ;rutina se zacykli, odchod jedine resetem CPU!     
      call errstop      	   ;vypnuti chyboveho citace v pripade prekroceni
                             ;casu na odpoved z modulu  
error1:
      mov a,ERRCODE
      mov R0,a
errcyk:	                     ;vlastni cyklus blikani (vyhodit zelenou!) 
	    clr LEDCER
	    clr LEDZEL
	    call CEKEJ250MS
	    setb LEDCER
	    setb LEDZEL
	    call CEKEJ250MS
	    djnz R0,errcyk         ;konec cyklu kratkych bliku
	    call CEKEJ250MS        ;nasleduje delsi pauza 
	    call CEKEJ250MS
	    call CEKEJ250MS
	    sjmp error1            ;a porad dokola...	    Ukonceni jedine RESETem
;*******************************************************************************

trojblik:      
      mov R0, #3              ;3x kratke bliknuti zelenou LED = inicializace OK 
c3blik:
      setb LEDCER
      clr  LEDZEL
      call cekej250ms
      setb LEDCER
      setb LEDZEL
      call cekej250ms
      djnz R0,c3blik
      sjmp standby            ;skok na pohotovostni rezim (preskoc idle mod)
      
;*******************************************************************************
idlemode:
      setb LEDZEL             ;zhasni LED
      setb LEDCER
      mov IE,#10000101b       ;povol preruseni INT0 a INT1 a globalne
      mov PCON,#00000001b     ;prechod do IDLE modu se snizenou spotrebou
      ;zde hajame a cekame na probuzeni pomoci INT0 (RING) ci INT1 (VIDLICE)
      nop                     ;sem se skace po probuzeni, pokracujeme na standby
      nop
      nop
      clr LEDZEL
      call CEKEJ20MS          ;chvilku pockame aby bylo videt bliknuti zel. LED
      
;************ Pohotovostni stav telefonu ***************************************	    
standby:	                   ;telefon je pripraven a je v klidu
      clr LEDZEL             ;sviti zelena LED
      setb LEDCER
      mov TOF1,#000h         ;nuluj merice casu do IDLE modu
      mov TOF2,#000h
      mov TOF3,#000h
      
standby2:    
      mov COUNT,#POCPRUCH        ;nastavime pocet pruchodu 
standby21:        
      jnb RING, zvoneni          ;prichozi zvoneni? ANO, jdeme na prijem hovoru
	    jnb LOWBAT, baterie        ;vybita baterie?, ANO, budeme to signalizovat
	    mov a,TOF1                 ;inkrementace merice casu do IDLE modu 
	    add a,#1                   ;TOF1 = TOF1 + 1
	    mov TOF1,a
	    mov a,TOF2
	    addc a,#0                  ;TOF2 = TOF2 + 0 + C
	    mov TOF2,a
	    mov a,TOF3
	    addc a,#0                  ;TOF3 = TOF3 + 0 + C
	    mov TOF3,a
	    xrl a,#20
	    jz idlemode                ;TOF3 dosahl zadane hodnoty, jdeme na IDLE
	    ;jc idlemode                ;TOF3 pretekl, jdeme hajat do IDLE modu
	    jb VIDLICE,standby2        ;zvednute sluchatko? ANO, budeme vytacet cislo
	    djnz COUNT, standby21
	    sjmp zadanicisla           ;jinak porad cekame, co vlastne bude
	    

baterie:                      ;Vybita baterie
      mov R0,#000h            ;chvilku pockame a otestujeme to znova
      djnz R0,$               ;kvuli hysterezi komparatoru 
      jb LOWBAT, standby      ;porad hlasi nizke napeti?, NE, vrat se
      clr LEDZEL              ;rozsvit LED oranzove 
      clr LEDCER
      sjmp standby2           ;a vrat se. Vim, dost primitivni, ale staci to....
	    
	    
;************* Prijem hovoru ***************************************************	       
zvoneni:	                       ;Nekdo vola!
      call CEKEJ250MS	           ;ignorace prichozi SMS tim, ze pockame >120 ms
      jb RING, standby           ;porad zvoni? Pak jde o hovor, jinak se vracime
      call zvonstart             ;Hovor: nastartujeme casovac menice pro zvonek
      
zcykl1:                          ;cekame na zvednuti sluchatka
      mov COUNT,#POCPRUCH        ;nastavime poc. pruchodu, pro osetreni kontaktu 
zcykl11:      
      jb RING, zvoneniend        ;pokud mezitim prestane zvonit, koncime
      jb VIDLICE, zcykl1         ;jinak cekame, jestli se nekdo bude obtezovat
      djnz COUNT, zcykl11        ;hovor prijmout zvednutim sluchatka. 
      call zvonstop              ;ANO, vypneme zvonek
      mov dptr,#TXT_ATA          ;a posleme prikaz na ATA na prijmuti hovoru 
	    call poslitext
	    call newline
      
zcykl2:                          ;Pak uz jen zase cekame, az se dotycny vykeca 
                                 ;a polozi sluchatko... 
      mov COUNT,#POCPRUCH        ;nastavime poc. pruchodu, pro osetreni kontaktu
zcykl21:      
      jnb VIDLICE, zcykl2        ;opet osetreni zakmitu
      djnz COUNT, zcykl21     
      mov dptr,#TXT_ATH          ;abychom mohli odeslat prikaz zaveseni ATH.
	    call poslitext             ;Prikaz odesleme bez ohledu na to, kdo zavesil
	    call newline               ;prvni. Nicemu to nevadi.
      jmp standby                ;A jdeme do pohotovostniho stavu
      
zvoneniend:
      call zvonstop              ;nikdo to u nas nebere? Vypneme zvonek
      jmp standby                ;a jdeme do pohotovostniho stavu

;**************** Volani *******************************************************
zadanicisla:                     ;budeme vytacet cislo na rotacaku:-) 
      mov CIFCOUNT,#000h         ;tak se na to hezky nachystame
      mov POCCIFER,#TEL9         ;standardni delka tel. cisla je 9 cifer
      clr SEZNAM                 ;vytacime klasicke tel. primo
      setb LEDZEL                ;zhasneme LEDky 
      setb LEDCER
           
ciselnice:                       ;a jedeeem....   
      mov CIFRA,#0  
      ;jb VIDLICE, cerr2          ;zmacknuta vidlice? skocime na osetreni     
      mov a,P3                   ;nejprve overime, ze ciselnice je v klidu 
      anl a,#C_MASK
      xrl a,#C_KLID
      jnz ciselnice              ;jestli ne, tak si na to pockame 
      
      mov STAVOLD,#C_KLID        ;nastavime klidovy stav.... 
      mov CISOLD,#C_KLID
      
cisnewcyk:                       ;...a spustime protizakmitovy cyklus cteni 
      mov COUNT,#POCPRUCH        ;nastavime pocet pruchodu 

ciscyk:                             ;doplnit reakci na RING? Zatim NE....
      jb VIDLICE, cerr2             ;zmacknuta vidlice? skocime na osetreni
      mov a,P3                      ;jinak cteme port ciselnice
      anl a,#C_MASK
      mov CISNEW,a
      xrl a,CISOLD
      jz ciscount
      mov CISOLD,CISNEW
      sjmp cisnewcyk 
      
ciscount:
      djnz COUNT,ciscyk
      
      mov a,CISNEW
      xrl a,#C_ZAK                   ;tenhle stav by vubec nemel nastat 
      jz cisnewcyk                   ;ale jestli jo, ignorujeme ho
       
      mov STAVNEW,CISNEW              
      mov a,STAVOLD
      xrl a,STAVNEW
      jz cisnewcyk
      mov STAVOLD,STAVNEW
      
      mov a,STAVNEW
      xrl a,#C_KLID
      jz ukazcislo
      
      mov a,STAVNEW
      xrl a,#C_NATAH
      jz cisnatah
      
      mov a,STAVNEW
      xrl a,#C_PULS
      jz cispuls
      
      jmp cisnewcyk
           
ukazcislo:                    ;Ciselnice se dotocila, zkontrolujeme cifru     
      mov a,CIFRA
      xrl a,#0                ;Je-li cifra nulova, je to chyba!   
      jz cerr1                ;Jinak testujeme dal 
      cjne CIFRA,#11,($+3)    ;Je-li CIFRA >= 11, je to chyba!     
      jc  ukazcislo2          ;CIFRA je mensi, nez 11, budeme ukladat do RAM 

cerr1:                        ;Chyba cteni ciselnice  (cifra = 0, cifra > 10)  
      clr LEDZEL              ;Signalizujeme kratkym blikanim oranzove  
      clr LEDCER
      call cekej250ms
      jb VIDLICE, cerr2       ;ale pritom testujeme, zmacknuti vidlice
      setb LEDZEL
      setb LEDCER
      call cekej250ms
      jb vidlice, cerr2       ;je-li vidlice zmacknuta, vyskocime z chyby
      sjmp cerr1            
cerr2:                          
      jnb VIDLICE, cerr2        ;trochu oosetrime kontakt a overime 
      nop                       ;spravne polozeni sluchatka
      nop
      jnb VIDLICE, cerr2        ;opet primitivni osetreni zakmitu
      nop                       ;na vidlici se predpoklada mzikovy spinac!
      nop
      jnb VIDLICE, cerr2           
      jmp standby
      
      
ukazcislo2:      
      setb LEDCER
      setb LEDZEL
      mov a,CIFRA             ;Ale nejprve z 10 (odpovida 0 na ciselnici)
      xrl a,#10               ;udelame tu spravnou nulu. Jinak cisla odpovidaji
      jnz ulozcifru           ;1-9 na ciselnici = 1-9 inkrementaci CIFRY
      mov CIFRA,#0
      
ulozcifru:                    ;ulozime cifru do RAM a vytvorime tak cele
      mov a,CIFCOUNT          ;telefonni cislo
      add a,#TELBASE          ;jednotlive cifry ukladame na adresy 10h - 18h
      mov R0,a
      mov a,#'0'              ;ASCII kod znaku nula
      add a,CIFRA             ;ulozime cifru rovnou v ASCII kodu
      mov @R0,a               ;pro pozdejsi prime odeslani do modulu seriakem
      
      mov a,CIFCOUNT          ;byla vytocena prvni cifra z tel cisla?
      xrl a,#0
      jz testujcifru          ;ANO, jaka to asi byla?  
      
      mov a,CIFCOUNT          ;NE, byla to uz druha cifra?
      xrl a,#1
      jnz ulozcifru2
      jnb SEZNAM, ulozcifru2  ;a jsme v rezimu seznamu?
      sjmp vypiscislo         ;NE, jedeme dal
      
testujcifru:      
      mov POCCIFER,#TEL9      ;ANO, nastavime pocet cifer na 9
      mov a,CIFRA             ;Je prvni cifra = 1 ?
      xrl a,#1
      jnz cifrasez            ;NE, ponechame pocet cifer na 9 a testujeme seznam
      mov POCCIFER,#TEL3      ;ANO, pocet cifer bude 3 - tisnove volani!
      sjmp ulozcifru2
      
cifrasez:
      clr SEZNAM
      mov a,CIFRA             ;Je prvni cifra = 0 ?
      xrl a,#0
      jnz ulozcifru2          ;NE, ponechame pocet cifer na 9 a jedeme dal
      setb SEZNAM             ;ANO, budeme volit ze seznamu
            

ulozcifru2:      
      inc CIFCOUNT            ;inkrementujeme pocitadlo vytocenych cifer
      mov a,CIFCOUNT          ;uz bylo vytoceno cele cislo?  
      xrl a,POCCIFER
      jnz cisend              ;NE, pokracujeme ve vytaceni
      jmp vypiscislo          ;ANO, posleme ho do GSM modulu

cisend:      
      jmp ciselnice           ;Pokracujeme ve vytaceni
      

cisnatah:                     ;Osetreny stav kontaktu pri natahovani
      setb LEDZEL             ;jen signalizujeme LEDkou
      clr LEDCER
      jmp cisnewcyk
      
cispuls:                      ;Osetreny stav kontaktu pri pulsu
      inc CIFRA               ;inkrementujeme vytacenou cifru
      jmp cisnewcyk   
      
        
vypiscislo:                  ;posleme hotove cislo do modulu a zahajime hovor
      clr LEDZEL             ;Signalizujeme kratkym bliknutim zelene  
      call cekej250ms
      setb LEDZEL
      call cekej250ms
      clr LEDCER             ;a pak trvalym svitem oranzove 
      clr LEDZEL
      jb SEZNAM, telsez      ;je-li volba ze seznamu, skoc
      add a,#TELBASE         ;nastavime ukazatel na zacatek ulozeneho cisla
      mov R0,a
      mov dptr,#TXT_ATD      ;odesleme prefix ATD00420 do modulu 
	    call poslitext
      mov b,POCCIFER         ;a v cyklu pak odesleme jednotlive cifry 
vypiscyk:                    ;z ulozeneho tel. cisla
      mov a,@R0
      mov R1,a
      call posliznak
      inc R0
      djnz b, vypiscyk
      mov R1,#';'            ;nakonec musi nasledovat strednik! 
      call posliznak
      call newline           ;a pochopitelne ENTER 
      sjmp endhov
      
telsez:
      mov a, CIFRA
      mov b, #19
      mul ab 
      mov dptr,#TXT_SEZ      ;odesleme cislo ze seznamu do modulu
      add a, DPL
      mov DPL,a
      mov a, #0
      addc a, DPH
      mov DPH,a 
	    call poslitext
	    call newline           ;a pochopitelne ENTER 
       
      
;zde probiha vlastni vytaceni cisla GSM modulem a pokud to na druhe strane
;nekdo zvedne tak i vlastni hovor. Nadale muzeme jen cekat na polozeni sluchatka      
      
endhov:                         ;cekani na polozeni sluchatka
      mov COUNT,#POCPRUCH       ;nastavime poc. pruchodu, pro osetreni kontaktu
endhov2:      
      jnb VIDLICE, endhov        ;opet osetreni zakmitu
      djnz COUNT, endhov2   
      mov dptr,#TXT_ATH        ;odesleme prikaz k zaveseni hovoru (ATH) modulem
	    call poslitext           ;prikaz odeslem bez ohledu na to, kdo zavesil
	    call newline             ;prvni. Nicemu to nevadi.
	    setb LEDZEL
	    setb LEDCER
	    jmp standby              ;a vracime se do pohotovostniho stavu 
        
        
;*******************************************************************************      	    
;********************************** Rutiny obsluhy preruseni *******************

; Rutina obsluhy vnejsich preruseni INT0, INT1 (spolecna pro obe)
; Zajisti jen probuzeni z IDLE modu a zakaze dalsi preruseni
inter0:
inter1:
     mov IE,#00000000b       ;zakaz vsechna preruseni
     clr IE0                 ;smaz priznaky preruseni
     clr IE1                                        
	   MOV PCON,#00000000b     ;SMOD = 0, IDL=0
	   reti                    ;jsme vzhuru a vracime se do prace:-)
     
; Rutina obsluhy preruseni od citace 0.
; Odzkouseno a chodi to !
citac0: 
      CLR TR0                   ;zastav casovac  
	    CLR TF0                   ;a vynuluj priznak preruseni 
      jnb ERRTIME, cit_zvon     ;je-li casovac pouzit pro casovani chyby
      djnz ERRCOUNT,pomerr      ;a byl-li prekrocen max. pocet preteceni,
      jmp error                 ;skoc na indikaci chyby, jejiz kod je v ERRCODE
pomerr: 
      setb TR0                  ;jinak restartuj casovac v modu chyby
      reti      
      
cit_zvon:                       ;casovac je v modu zvoneni, obslouzime ho  
      PUSH PSW                  
	    PUSH ACC
	    setb TRAFO1               ;vypneme obe vinuti
      setb TRAFO2
      inc ZSTATE                ;inkrementujeme stav
      mov a,ZSTATE
      xrl a,#4                  ;je-li stav = 4, pak stav = 0    
      jnz cit0_1
      mov ZSTATE,#0
      
cit0_1:                         ;rozlisime aktivni stavy 1 a 3
      mov a,ZSTATE          
      xrl a,#1
      jz trpuls1                ;jedno vinuti
      mov a,ZSTATE
      xrl a,#3
      jz trpuls2                ;druhe vinuti
      
deadtime:                       ;stav 0 a 2 je deadtime (obe vinuti vypnuta)
      MOV TL0, #LOW(PULS1)        
	    MOV TH0, #HIGH(PULS1)
	    inc TOF1
	    mov a,TOF1
	    xrl a,#ZVONENICYK         ;prekrocen pocet pulperiod? 
      jnz cit0_end              ;NE, koncime
      mov TOF1,#0               ;ANO, vynulujeme jejich pocitadlo 
      cpl ZVON                  ;zmenime zvonek/pauza
      mov c,ZVON
      mov LEDZEL,c              ;signalizujeme na LED
      jmp cit0_end              ;a koncime
        
trpuls1:                        ;1. vinuti trafa na zem
	    MOV TL0, #LOW(PULS10)
	    MOV TH0, #HIGH(PULS10)
	    jb ZVON,cit0_end
	    clr TRAFO1
      jmp cit0_end 

trpuls2:                        ;2. vinuti trafa na zem
      MOV TL0, #LOW(PULS10)
	    MOV TH0, #HIGH(PULS10)
	    jb ZVON,cit0_end
	    clr TRAFO2 
      
cit0_end: 	  
      setb TR0                   ;restartujeme casovac v modu zvonku 
	    POP ACC
	    POP PSW
	    RETI

;*******************************************************************************
;******************** podprogramy **********************************************	

zvonstart:
      clr ERRTIME              ;tohle je casovac zvoneni, ne chyby
      mov ZSTATE,#0            ;pocatecni stav = 0
	    mov TOF1,#0              ;nulovani pocitadla pulperiod (zvoneni/pauza) 
	    clr ZVON                  
	    clr LEDZEL
      setb LEDCER    
	    MOV TL0, #LOW(PULS1)     ;zaciname deadtimem 
	    MOV TH0, #HIGH(PULS1)  
	    clr TF0                  ;nuluj priznak preruseni timeru 0
	    SETB ET0                 ;povol preruseni od timeru 0
      SETB EA                  ;povol preruseni globalne
	    SETB TR0                 ;spust timer 0	   
      ret
      
zvonstop:                      ;zastaveni casovace menice zvonku a uklid  
      clr TR0                  ;po jeho cinnosti
      clr TF0
      clr ET0
      clr EA
      setb TRAFO1
      setb TRAFO2
      setb LEDZEL
      ret
      
errstart:                      ;start chyboveho casovace
      setb ERRTIME             ;priznak toho, ze se jedna o casovani chyby
      mov ERRCOUNT,#3          ;pocet moznych preteceni chyboveho casovace
	    MOV TL0, #000h           ;max. mozna perioda casovace
	    MOV TH0, #000h
      clr TF0                  ;nuluj priznak preruseni timeru 0
	    SETB ET0                 ;povol preruseni od timeru 0
      SETB EA                  ;povol preruseni globalne
	    SETB TR0                 ;spust timer 0	   
      ret
      
errstop:                       ;zastaveni chyboveho casovace a uklid 
      clr TR0                  ;po jeho cinnosti
      clr TF0
      clr ET0
      clr EA
      clr ERRTIME
      ret      

      
;odradkovani na terminalu (/r/n)	 =  ENTER!       
newline:                         
          mov	R1,#00Dh           ;CR (/r)
	        call	posliznak 
	        mov	R1,#00Ah           ;LF (/n)
	        call	posliznak 
	        ret	        


;podprogram pro odeslani jednoho znaku seriakem. Znak predan v R1.
posliznak:                        ;podprogram pro odeslani znaku seriakem
          clr ti                  ;nulovn pznaku dokonenho odvysln
          mov sbuf,R1             ;vysln obsahu R1
          jnb ti,$                ;ekn na odvysln
          clr ti                  ;nulovn pznaku dokonenho odvysln
          ret	      
 
          
;podprogram pro cteni jednoho znaku se seriaku. Znak vracen v R1.
;Pred prvnim pouzitim nutno vynulovat Ri!          
ctiznak:                          ;podprogram pro cteni jednoho znaku se seriaku          
          jnb   ri,$		          ;cekej na znak z linky	
	        mov	  a,sbuf		        ;nacti hodnotu z UARTU do akumulatoru
	        clr	ri		              ;nuluj flag RX
	        mov	R1,A		            ;prijatou hodnotu uloz do R1
          ret       
          
          
;odesle retezec ASCII znaku ukonceny @ BEZ odradkovani pred a po textu!	     	
poslitext:                    ;odesle retezec ASCII znaku ukonceny @
                              ;pocatek retezce predan v DPTR         
          mov a,#000h
          movc a,@A+DPTR        
          mov R1,a
          xrl a,#'@'          ;je to @? ANO, koncime
          jz endtext
          call posliznak
          inc DPTR
          jmp poslitext
endtext:  
          ret  
 
          
;Cekani na OK z GSM modulu. Krome spravnosti znaku je jeste nezavisle hlidana
;max doba na prijeti odpovedi pomoci chyboveho casovace!          
isok:     
      clr ri                   ;nulujeme priznak predchoziho prijmu
      clr ERRBIT               ;nulujeme priznak chyby
isok2:	    
	    call ctiznak             ;cteme seriak a cekame na znak "O" 
	    mov a,R1
	    xrl a,#'O'
	    jnz isok2
      call ctiznak             ;pokud prijde, dalsim znakem musi byt "K"
	    mov a,R1
	    xrl a,#'K'
	    jz isok3
      setb ERRBIT              ;neni to K, nastavime priznak chyby
isok3:         
      ret                      ;a vracime se
      

;Cekani na potvrzeni registrace do GSM site (0,1).
;Krome spravnosti znaku je jeste nezavisle hlidana
;max doba na prijeti odpovedi pomoci chyboveho casovace!         
isgsm:      
      clr ri                   ;nulujeme priznak predchoziho prijmu
      clr ERRBIT               ;nulujeme priznak chyby
isgsm2:	    
	    call ctiznak             ;cteme seriak a cekame na znak "0" 
	    mov a,R1
	    xrl a,#'0'
	    jnz isgsm2
	    call ctiznak             ;tohle by mela byt carka
	    call ctiznak             ;a treti znak musi byt "1"
	    mov a,R1
	    xrl a,#'1'
	    jz isgsm3
      setb ERRBIT              ;neni to 1, nastavime priznak chyby
isgsm3:
      ret                      ;a vracime se
   
          
;cekaci smycky pro generovani casoveho zpozdeni          
CEKEJ2S:			               ;cas. prodleva  cca 2 sec!
	mov	TOF1,#0
  mov TOF2,#0          
	mov	TOF3,#16
PCEKEJ2S:	 
  djnz	TOF1,PCEKEJ2S      ;DJNZ = 2 strojni cykly po 1 mikrosekunde
  djnz  TOF2,PCEKEJ2S	     ;prodleva = 256 x 256 x 16 x 2 mikrosekund 
	djnz	TOF3,PCEKEJ2S
	Ret               
       
       
CEKEJ250MS:			              ;cas. prodleva  cca 260 msec!
	mov	TOF1,#0
  mov TOF2,#0          
	mov	TOF3,#2
PCEKEJ250MS:	 
  djnz	TOF1,PCEKEJ250MS      ;DJNZ = 2 strojni cykly po 1 mikrosekunde
  djnz  TOF2,PCEKEJ250MS	    ;prodleva = 256 x 256 x 2 x 2 mikrosekund 
	djnz	TOF3,PCEKEJ250MS
	Ret  
	
	
CEKEJ20MS:			              ;cas. prodleva  cca 20 msec!
	mov	TOF1,#0
  mov TOF2,#40          
PCEKEJ20MS:	 
  djnz	TOF1,PCEKEJ20MS      ;DJNZ = 2 strojni cykly po 1 mikrosekunde
  djnz  TOF2,PCEKEJ20MS	     ;prodleva = 256 x 40 x  2 mikrosekund 
	Ret  	
            
       
;-------------------------------------------------------------------------------          
;texty pro odesilani do GSM modulu seriakem. Musi vzdy koncit znakem @!
TXT_GSM:  db 'AT+CREG?@'      ;kontrola registrace do GSM site
;TXT_CHA1: db 'AT+QAUDCH=1@'   ;audio na jack konektor 
TXT_CHA0: db 'AT+QAUDCH=0@'   ;audio na piny J2 
TXT_ATD:  db 'ATD00420@'      ;vytoc cislo
TXT_ATH:  db 'ATH@'           ;zavesit
TXT_ATA:  db 'ATA@'           ;prijmout hovor
TXT_ATE0: db 'ATE0@'          ;vypnuti echa
TXT_AT:   db 'AT@'            ;prikaz AT, kvuli autobaud funkci

;************ Predvolena telefonni cisla ***************************************
;
;Cisla jednotlivych voleb se zadavaji vcetne prikazu ATD a mezinarodni predvolby
;Koncit musi strednikem a nasleduje @ pro ukonceni prenosu
;Celkova delka retezce vcetne @ musi byt presne 19 znaku!
;Za cislem je nutny strednik.
;Je-li potreba cislo kratsi, doplni se za @ mezery.
;Delsi cislo nelze vlozit bez upravy programu!

TXT_SEZ:  db 'ATD00420123123123;@'         ;Pametove misto 0 (volba 00)
          db 'ATD00420123123123;@'         ;Pametove misto 1 (volba 01)
          db 'ATD00420123123123;@'         ;Pametove misto 2 (volba 02)
          db 'ATD00420123123123;@'         ;Pametove misto 3 (volba 03)
          db 'ATD00420123123123;@'         ;Pametove misto 4 (volba 04)
          db 'ATD00420123123123;@'         ;Pametove misto 5 (volba 05)
          db 'ATD00420123123123;@'         ;Pametove misto 6 (volba 06)
          db 'ATD00420123123123;@'         ;Pametove misto 7 (volba 07)
          db 'ATD00420123123123;@'         ;Pametove misto 8 (volba 08)
          db 'ATD00420123123123;@'         ;Pametove misto 9 (volba 09)
      
END
